@@ -15,6 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#include <algorithm>
 #include <exception>
 #include <memory>
 #include <sstream>
@@ -22,22 +23,6 @@
 #include <utility>
 #include <vector>
 
-// This boost/asio/io_context.hpp include is needless for no MinGW
-// build.
-//
-// This is for including boost/asio/detail/socket_types.hpp before any
-// "#include <windows.h>". boost/asio/detail/socket_types.hpp doesn't
-// work if windows.h is already included. boost/process.h ->
-// boost/process/args.hpp -> boost/process/detail/basic_cmd.hpp
-// includes windows.h. boost/process/args.hpp is included before
-// boost/process/async.h that includes
-// boost/asio/detail/socket_types.hpp implicitly is included.
-#include <boost/asio/io_context.hpp>
-// We need BOOST_USE_WINDOWS_H definition with MinGW when we use
-// boost/process.hpp. See ARROW_BOOST_PROCESS_COMPILE_DEFINITIONS in
-// cpp/cmake_modules/BuildUtils.cmake for details.
-#include <boost/process.hpp>
-
 #include <gmock/gmock-matchers.h>
 #include <gtest/gtest.h>
 
@@ -55,6 +40,7 @@
 #include <aws/core/Version.h>
 #include <aws/core/auth/AWSCredentials.h>
 #include <aws/core/auth/AWSCredentialsProvider.h>
+#include <aws/core/client/CoreErrors.h>
 #include <aws/core/client/RetryStrategy.h>
 #include <aws/core/utils/logging/ConsoleLogSystem.h>
 #include <aws/s3/S3Client.h>
@@ -72,7 +58,7 @@
 #include "arrow/status.h"
 #include "arrow/testing/future_util.h"
 #include "arrow/testing/gtest_util.h"
-#include "arrow/testing/util.h"
+#include "arrow/testing/matchers.h"
 #include "arrow/util/async_generator.h"
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/future.h"
@@ -93,7 +79,45 @@ using ::arrow::fs::internal::ErrorToStatus;
 using ::arrow::fs::internal::OutcomeToStatus;
 using ::arrow::fs::internal::ToAwsString;
 
-namespace bp = boost::process;
+// Use "short" retry parameters to make tests faster
+static constexpr int32_t kRetryInterval = 50;      /* milliseconds */
+static constexpr int32_t kMaxRetryDuration = 6000; /* milliseconds */
+
+::testing::Environment* s3_env = ::testing::AddGlobalTestEnvironment(new S3Environment);
+
+::testing::Environment* minio_env =
+    ::testing::AddGlobalTestEnvironment(new MinioTestEnvironment);
+
+MinioTestEnvironment* GetMinioEnv() {
+  return ::arrow::internal::checked_cast<MinioTestEnvironment*>(minio_env);
+}
+
+class ShortRetryStrategy : public S3RetryStrategy {
+ public:
+  bool ShouldRetry(const AWSErrorDetail& error, int64_t attempted_retries) override {
+    if (error.message.find(kFileExistsMessage) != error.message.npos) {
+      // Minio returns "file exists" errors (when calling mkdir) as internal errors,
+      // which would trigger spurious retries.
+      return false;
+    }
+    return IsRetryable(error) && (attempted_retries * kRetryInterval < kMaxRetryDuration);
+  }
+
+  int64_t CalculateDelayBeforeNextRetry(const AWSErrorDetail& error,
+                                        int64_t attempted_retries) override {
+    return kRetryInterval;
+  }
+
+  bool IsRetryable(const AWSErrorDetail& error) const {
+    return error.should_retry || error.exception_name == "XMinioServerNotInitialized";
+  }
+
+#ifdef _WIN32
+  static constexpr const char* kFileExistsMessage = "file already exists";
+#else
+  static constexpr const char* kFileExistsMessage = "file exists";
+#endif
+};
 
 // NOTE: Connecting in Python:
 // >>> fs = s3fs.S3FileSystem(key='minio', secret='miniopass',
@@ -121,11 +145,6 @@ namespace bp = boost::process;
 
 class AwsTestMixin : public ::testing::Test {
  public:
-  // We set this environment variable to speed up tests by ensuring
-  // DefaultAWSCredentialsProviderChain does not query (inaccessible)
-  // EC2 metadata endpoint
-  AwsTestMixin() : ec2_metadata_disabled_guard_("AWS_EC2_METADATA_DISABLED", "true") {}
-
   void SetUp() override {
 #ifdef AWS_CPP_SDK_S3_NOT_SHARED
     auto aws_log_level = Aws::Utils::Logging::LogLevel::Fatal;
@@ -144,7 +163,6 @@ class AwsTestMixin : public ::testing::Test {
   }
 
  private:
-  EnvVarGuard ec2_metadata_disabled_guard_;
 #ifdef AWS_CPP_SDK_S3_NOT_SHARED
   Aws::SDKOptions aws_options_;
 #endif
@@ -155,28 +173,45 @@ class S3TestMixin : public AwsTestMixin {
   void SetUp() override {
     AwsTestMixin::SetUp();
 
-    ASSERT_OK(minio_.Start());
+    // Starting the server may fail, for example if the generated port number
+    // was "stolen" by another process. Run a dummy S3 operation to make sure it
+    // is running, otherwise retry a number of times.
+    Status connect_status;
+    int retries = kNumServerRetries;
+    do {
+      ASSERT_OK(InitServerAndClient());
+      connect_status = OutcomeToStatus("ListBuckets", client_->ListBuckets());
+    } while (!connect_status.ok() && --retries > 0);
+    ASSERT_OK(connect_status);
+  }
 
+  void TearDown() override {
+    client_.reset();  // Aws::S3::S3Client destruction relies on AWS SDK, so it must be
+                      // reset before Aws::ShutdownAPI
+    AwsTestMixin::TearDown();
+  }
+
+ protected:
+  Status InitServerAndClient() {
+    ARROW_ASSIGN_OR_RAISE(minio_, GetMinioEnv()->GetOneServer());
     client_config_.reset(new Aws::Client::ClientConfiguration());
-    client_config_->endpointOverride = ToAwsString(minio_.connect_string());
+    client_config_->endpointOverride = ToAwsString(minio_->connect_string());
     client_config_->scheme = Aws::Http::Scheme::HTTP;
-    client_config_->retryStrategy = std::make_shared<ConnectRetryStrategy>();
-    credentials_ = {ToAwsString(minio_.access_key()), ToAwsString(minio_.secret_key())};
+    client_config_->retryStrategy =
+        std::make_shared<ConnectRetryStrategy>(kRetryInterval, kMaxRetryDuration);
+    credentials_ = {ToAwsString(minio_->access_key()), ToAwsString(minio_->secret_key())};
     bool use_virtual_addressing = false;
     client_.reset(
         new Aws::S3::S3Client(credentials_, *client_config_,
                               Aws::Client::AWSAuthV4Signer::PayloadSigningPolicy::Never,
                               use_virtual_addressing));
+    return Status::OK();
   }
 
-  void TearDown() override {
-    ASSERT_OK(minio_.Stop());
+  // How many times to try launching a server in a row before decreeing failure
+  static constexpr int kNumServerRetries = 3;
 
-    AwsTestMixin::TearDown();
-  }
-
- protected:
-  MinioTestServer minio_;
+  std::shared_ptr<MinioTestServer> minio_;
   std::unique_ptr<Aws::Client::ClientConfiguration> client_config_;
   Aws::Auth::AWSCredentials credentials_;
   std::unique_ptr<Aws::S3::S3Client> client_;
@@ -301,28 +336,35 @@ TEST_F(S3OptionsTest, FromAssumeRole) {
 class S3RegionResolutionTest : public AwsTestMixin {};
 
 TEST_F(S3RegionResolutionTest, PublicBucket) {
-  ASSERT_OK_AND_EQ("us-east-2", ResolveBucketRegion("ursa-labs-taxi-data"));
+  ASSERT_OK_AND_EQ("us-east-2", ResolveS3BucketRegion("voltrondata-labs-datasets"));
 
   // Taken from a registry of open S3-hosted datasets
   // at https://github.com/awslabs/open-data-registry
-  ASSERT_OK_AND_EQ("eu-west-2", ResolveBucketRegion("aws-earth-mo-atmospheric-ukv-prd"));
+  ASSERT_OK_AND_EQ("eu-west-2",
+                   ResolveS3BucketRegion("aws-earth-mo-atmospheric-ukv-prd"));
   // Same again, cached
-  ASSERT_OK_AND_EQ("eu-west-2", ResolveBucketRegion("aws-earth-mo-atmospheric-ukv-prd"));
+  ASSERT_OK_AND_EQ("eu-west-2",
+                   ResolveS3BucketRegion("aws-earth-mo-atmospheric-ukv-prd"));
 }
 
 TEST_F(S3RegionResolutionTest, RestrictedBucket) {
-  ASSERT_OK_AND_EQ("us-west-2", ResolveBucketRegion("ursa-labs-r-test"));
+  ASSERT_OK_AND_EQ("us-west-2", ResolveS3BucketRegion("ursa-labs-r-test"));
   // Same again, cached
-  ASSERT_OK_AND_EQ("us-west-2", ResolveBucketRegion("ursa-labs-r-test"));
+  ASSERT_OK_AND_EQ("us-west-2", ResolveS3BucketRegion("ursa-labs-r-test"));
 }
 
 TEST_F(S3RegionResolutionTest, NonExistentBucket) {
-  auto maybe_region = ResolveBucketRegion("ursa-labs-non-existent-bucket");
+  auto maybe_region = ResolveS3BucketRegion("ursa-labs-non-existent-bucket");
   ASSERT_RAISES(IOError, maybe_region);
   ASSERT_THAT(maybe_region.status().message(),
               ::testing::HasSubstr("Bucket 'ursa-labs-non-existent-bucket' not found"));
 }
 
+TEST_F(S3RegionResolutionTest, InvalidBucketName) {
+  ASSERT_RAISES(Invalid, ResolveS3BucketRegion("s3:bucket"));
+  ASSERT_RAISES(Invalid, ResolveS3BucketRegion("foo/bar"));
+}
+
 ////////////////////////////////////////////////////////////////////////////
 // S3FileSystem region test
 
@@ -377,36 +419,46 @@ class TestS3FS : public S3TestMixin {
  public:
   void SetUp() override {
     S3TestMixin::SetUp();
+    // Most tests will create buckets
+    options_.allow_bucket_creation = true;
+    options_.allow_bucket_deletion = true;
     MakeFileSystem();
     // Set up test bucket
     {
       Aws::S3::Model::CreateBucketRequest req;
       req.SetBucket(ToAwsString("bucket"));
-      ASSERT_OK(OutcomeToStatus(client_->CreateBucket(req)));
+      ASSERT_OK(OutcomeToStatus("CreateBucket", client_->CreateBucket(req)));
       req.SetBucket(ToAwsString("empty-bucket"));
-      ASSERT_OK(OutcomeToStatus(client_->CreateBucket(req)));
+      ASSERT_OK(OutcomeToStatus("CreateBucket", client_->CreateBucket(req)));
     }
     {
       Aws::S3::Model::PutObjectRequest req;
       req.SetBucket(ToAwsString("bucket"));
       req.SetKey(ToAwsString("emptydir/"));
-      ASSERT_OK(OutcomeToStatus(client_->PutObject(req)));
+      req.SetBody(std::make_shared<std::stringstream>(""));
+      ASSERT_OK(OutcomeToStatus("PutObject", client_->PutObject(req)));
       // NOTE: no need to create intermediate "directories" somedir/ and
       // somedir/subdir/
       req.SetKey(ToAwsString("somedir/subdir/subfile"));
       req.SetBody(std::make_shared<std::stringstream>("sub data"));
-      ASSERT_OK(OutcomeToStatus(client_->PutObject(req)));
+      ASSERT_OK(OutcomeToStatus("PutObject", client_->PutObject(req)));
       req.SetKey(ToAwsString("somefile"));
       req.SetBody(std::make_shared<std::stringstream>("some data"));
       req.SetContentType("x-arrow/test");
-      ASSERT_OK(OutcomeToStatus(client_->PutObject(req)));
+      ASSERT_OK(OutcomeToStatus("PutObject", client_->PutObject(req)));
+      req.SetKey(ToAwsString("otherdir/1/2/3/otherfile"));
+      req.SetBody(std::make_shared<std::stringstream>("other data"));
+      ASSERT_OK(OutcomeToStatus("PutObject", client_->PutObject(req)));
     }
   }
 
   void MakeFileSystem() {
-    options_.ConfigureAccessKey(minio_.access_key(), minio_.secret_key());
+    options_.ConfigureAccessKey(minio_->access_key(), minio_->secret_key());
     options_.scheme = "http";
-    options_.endpoint_override = minio_.connect_string();
+    options_.endpoint_override = minio_->connect_string();
+    if (!options_.retry_strategy) {
+      options_.retry_strategy = std::make_shared<ShortRetryStrategy>();
+    }
     ASSERT_OK_AND_ASSIGN(fs_, S3FileSystem::Make(options_));
   }
 
@@ -422,12 +474,30 @@ class TestS3FS : public S3TestMixin {
     ASSERT_THAT(got_metadata->sorted_pairs(), matcher);
   }
 
+  void AssertInfoAllBucketsRecursive(const std::vector<FileInfo>& infos) {
+    AssertFileInfo(infos[0], "bucket", FileType::Directory);
+    AssertFileInfo(infos[1], "bucket/emptydir", FileType::Directory);
+    AssertFileInfo(infos[2], "bucket/otherdir", FileType::Directory);
+    AssertFileInfo(infos[3], "bucket/otherdir/1", FileType::Directory);
+    AssertFileInfo(infos[4], "bucket/otherdir/1/2", FileType::Directory);
+    AssertFileInfo(infos[5], "bucket/otherdir/1/2/3", FileType::Directory);
+    AssertFileInfo(infos[6], "bucket/otherdir/1/2/3/otherfile", FileType::File, 10);
+    AssertFileInfo(infos[7], "bucket/somedir", FileType::Directory);
+    AssertFileInfo(infos[8], "bucket/somedir/subdir", FileType::Directory);
+    AssertFileInfo(infos[9], "bucket/somedir/subdir/subfile", FileType::File, 8);
+    AssertFileInfo(infos[10], "bucket/somefile", FileType::File, 9);
+    AssertFileInfo(infos[11], "empty-bucket", FileType::Directory);
+  }
+
   void TestOpenOutputStream() {
     std::shared_ptr<io::OutputStream> stream;
 
     // Nonexistent
     ASSERT_RAISES(IOError, fs_->OpenOutputStream("nonexistent-bucket/somefile"));
 
+    // URI
+    ASSERT_RAISES(Invalid, fs_->OpenOutputStream("s3:bucket/newfile1"));
+
     // Create new empty file
     ASSERT_OK_AND_ASSIGN(stream, fs_->OpenOutputStream("bucket/newfile1"));
     ASSERT_OK(stream->Close());
@@ -521,15 +591,25 @@ TEST_F(TestS3FS, GetFileInfoBucket) {
   AssertFileInfo(fs_.get(), "bucket/", FileType::Directory);
   AssertFileInfo(fs_.get(), "empty-bucket/", FileType::Directory);
   AssertFileInfo(fs_.get(), "nonexistent-bucket/", FileType::NotFound);
+
+  // URIs
+  ASSERT_RAISES(Invalid, fs_->GetFileInfo("s3:bucket"));
+  ASSERT_RAISES(Invalid, fs_->GetFileInfo("s3:empty-bucket"));
+  ASSERT_RAISES(Invalid, fs_->GetFileInfo("s3:nonexistent-bucket"));
 }
 
 TEST_F(TestS3FS, GetFileInfoObject) {
   // "Directories"
   AssertFileInfo(fs_.get(), "bucket/emptydir", FileType::Directory, kNoSize);
+  AssertFileInfo(fs_.get(), "bucket/otherdir", FileType::Directory, kNoSize);
+  AssertFileInfo(fs_.get(), "bucket/otherdir/1", FileType::Directory, kNoSize);
+  AssertFileInfo(fs_.get(), "bucket/otherdir/1/2", FileType::Directory, kNoSize);
+  AssertFileInfo(fs_.get(), "bucket/otherdir/1/2/3", FileType::Directory, kNoSize);
   AssertFileInfo(fs_.get(), "bucket/somedir", FileType::Directory, kNoSize);
   AssertFileInfo(fs_.get(), "bucket/somedir/subdir", FileType::Directory, kNoSize);
 
   // "Files"
+  AssertFileInfo(fs_.get(), "bucket/otherdir/1/2/3/otherfile", FileType::File, 10);
   AssertFileInfo(fs_.get(), "bucket/somefile", FileType::File, 9);
   AssertFileInfo(fs_.get(), "bucket/somedir/subdir/subfile", FileType::File, 8);
 
@@ -543,6 +623,11 @@ TEST_F(TestS3FS, GetFileInfoObject) {
   AssertFileInfo(fs_.get(), "bucket/somefile/", FileType::File, 9);
   AssertFileInfo(fs_.get(), "bucket/emptyd/", FileType::NotFound);
   AssertFileInfo(fs_.get(), "non-existent-bucket/somed/", FileType::NotFound);
+
+  // URIs
+  ASSERT_RAISES(Invalid, fs_->GetFileInfo("s3:bucket/emptydir"));
+  ASSERT_RAISES(Invalid, fs_->GetFileInfo("s3:bucket/otherdir"));
+  ASSERT_RAISES(Invalid, fs_->GetFileInfo("s3:bucket/somefile"));
 }
 
 TEST_F(TestS3FS, GetFileInfoSelector) {
@@ -572,10 +657,11 @@ TEST_F(TestS3FS, GetFileInfoSelector) {
   select.base_dir = "bucket";
   ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
   SortInfos(&infos);
-  ASSERT_EQ(infos.size(), 3);
+  ASSERT_EQ(infos.size(), 4);
   AssertFileInfo(infos[0], "bucket/emptydir", FileType::Directory);
-  AssertFileInfo(infos[1], "bucket/somedir", FileType::Directory);
-  AssertFileInfo(infos[2], "bucket/somefile", FileType::File, 9);
+  AssertFileInfo(infos[1], "bucket/otherdir", FileType::Directory);
+  AssertFileInfo(infos[2], "bucket/somedir", FileType::Directory);
+  AssertFileInfo(infos[3], "bucket/somefile", FileType::File, 9);
 
   // Empty "directory"
   select.base_dir = "bucket/emptydir";
@@ -607,7 +693,13 @@ TEST_F(TestS3FS, GetFileInfoSelector) {
   select.base_dir = "bucket/";
   ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
   SortInfos(&infos);
-  ASSERT_EQ(infos.size(), 3);
+  ASSERT_EQ(infos.size(), 4);
+
+  // URIs
+  select.base_dir = "s3:bucket";
+  ASSERT_RAISES(Invalid, fs_->GetFileInfo(select));
+  select.base_dir = "s3:bucket/somedir";
+  ASSERT_RAISES(Invalid, fs_->GetFileInfo(select));
 }
 
 TEST_F(TestS3FS, GetFileInfoSelectorRecursive) {
@@ -618,15 +710,9 @@ TEST_F(TestS3FS, GetFileInfoSelectorRecursive) {
   // Root dir
   select.base_dir = "";
   ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
-  ASSERT_EQ(infos.size(), 7);
+  ASSERT_EQ(infos.size(), 12);
   SortInfos(&infos);
-  AssertFileInfo(infos[0], "bucket", FileType::Directory);
-  AssertFileInfo(infos[1], "bucket/emptydir", FileType::Directory);
-  AssertFileInfo(infos[2], "bucket/somedir", FileType::Directory);
-  AssertFileInfo(infos[3], "bucket/somedir/subdir", FileType::Directory);
-  AssertFileInfo(infos[4], "bucket/somedir/subdir/subfile", FileType::File, 8);
-  AssertFileInfo(infos[5], "bucket/somefile", FileType::File, 9);
-  AssertFileInfo(infos[6], "empty-bucket", FileType::Directory);
+  AssertInfoAllBucketsRecursive(infos);
 
   // Empty bucket
   select.base_dir = "empty-bucket";
@@ -637,12 +723,17 @@ TEST_F(TestS3FS, GetFileInfoSelectorRecursive) {
   select.base_dir = "bucket";
   ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
   SortInfos(&infos);
-  ASSERT_EQ(infos.size(), 5);
+  ASSERT_EQ(infos.size(), 10);
   AssertFileInfo(infos[0], "bucket/emptydir", FileType::Directory);
-  AssertFileInfo(infos[1], "bucket/somedir", FileType::Directory);
-  AssertFileInfo(infos[2], "bucket/somedir/subdir", FileType::Directory);
-  AssertFileInfo(infos[3], "bucket/somedir/subdir/subfile", FileType::File, 8);
-  AssertFileInfo(infos[4], "bucket/somefile", FileType::File, 9);
+  AssertFileInfo(infos[1], "bucket/otherdir", FileType::Directory);
+  AssertFileInfo(infos[2], "bucket/otherdir/1", FileType::Directory);
+  AssertFileInfo(infos[3], "bucket/otherdir/1/2", FileType::Directory);
+  AssertFileInfo(infos[4], "bucket/otherdir/1/2/3", FileType::Directory);
+  AssertFileInfo(infos[5], "bucket/otherdir/1/2/3/otherfile", FileType::File, 10);
+  AssertFileInfo(infos[6], "bucket/somedir", FileType::Directory);
+  AssertFileInfo(infos[7], "bucket/somedir/subdir", FileType::Directory);
+  AssertFileInfo(infos[8], "bucket/somedir/subdir/subfile", FileType::File, 8);
+  AssertFileInfo(infos[9], "bucket/somefile", FileType::File, 9);
 
   // Empty "directory"
   select.base_dir = "bucket/emptydir";
@@ -656,6 +747,15 @@ TEST_F(TestS3FS, GetFileInfoSelectorRecursive) {
   ASSERT_EQ(infos.size(), 2);
   AssertFileInfo(infos[0], "bucket/somedir/subdir", FileType::Directory);
   AssertFileInfo(infos[1], "bucket/somedir/subdir/subfile", FileType::File, 8);
+
+  select.base_dir = "bucket/otherdir";
+  ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
+  SortInfos(&infos);
+  ASSERT_EQ(infos.size(), 4);
+  AssertFileInfo(infos[0], "bucket/otherdir/1", FileType::Directory);
+  AssertFileInfo(infos[1], "bucket/otherdir/1/2", FileType::Directory);
+  AssertFileInfo(infos[2], "bucket/otherdir/1/2/3", FileType::Directory);
+  AssertFileInfo(infos[3], "bucket/otherdir/1/2/3/otherfile", FileType::File, 10);
 }
 
 TEST_F(TestS3FS, GetFileInfoGenerator) {
@@ -673,15 +773,9 @@ TEST_F(TestS3FS, GetFileInfoGenerator) {
   // Root dir, recursive
   select.recursive = true;
   CollectFileInfoGenerator(fs_->GetFileInfoGenerator(select), &infos);
-  ASSERT_EQ(infos.size(), 7);
+  ASSERT_EQ(infos.size(), 12);
   SortInfos(&infos);
-  AssertFileInfo(infos[0], "bucket", FileType::Directory);
-  AssertFileInfo(infos[1], "bucket/emptydir", FileType::Directory);
-  AssertFileInfo(infos[2], "bucket/somedir", FileType::Directory);
-  AssertFileInfo(infos[3], "bucket/somedir/subdir", FileType::Directory);
-  AssertFileInfo(infos[4], "bucket/somedir/subdir/subfile", FileType::File, 8);
-  AssertFileInfo(infos[5], "bucket/somefile", FileType::File, 9);
-  AssertFileInfo(infos[6], "empty-bucket", FileType::Directory);
+  AssertInfoAllBucketsRecursive(infos);
 
   // Non-root dir case is tested by generic tests
 }
@@ -719,6 +813,9 @@ TEST_F(TestS3FS, CreateDir) {
 
   // Existing "file", should fail
   ASSERT_RAISES(IOError, fs_->CreateDir("bucket/somefile"));
+
+  // URI
+  ASSERT_RAISES(Invalid, fs_->CreateDir("s3:bucket/newdir2"));
 }
 
 TEST_F(TestS3FS, DeleteFile) {
@@ -736,6 +833,9 @@ TEST_F(TestS3FS, DeleteFile) {
   // "Directory"
   ASSERT_RAISES(IOError, fs_->DeleteFile("bucket/somedir"));
   AssertFileInfo(fs_.get(), "bucket/somedir", FileType::Directory);
+
+  // URI
+  ASSERT_RAISES(Invalid, fs_->DeleteFile("s3:bucket/somefile"));
 }
 
 TEST_F(TestS3FS, DeleteDir) {
@@ -746,29 +846,71 @@ TEST_F(TestS3FS, DeleteDir) {
   // Empty "directory"
   ASSERT_OK(fs_->DeleteDir("bucket/emptydir"));
   ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
-  ASSERT_EQ(infos.size(), 2);
+  ASSERT_EQ(infos.size(), 3);
   SortInfos(&infos);
-  AssertFileInfo(infos[0], "bucket/somedir", FileType::Directory);
-  AssertFileInfo(infos[1], "bucket/somefile", FileType::File);
+  AssertFileInfo(infos[0], "bucket/otherdir", FileType::Directory);
+  AssertFileInfo(infos[1], "bucket/somedir", FileType::Directory);
+  AssertFileInfo(infos[2], "bucket/somefile", FileType::File);
 
   // Non-empty "directory"
   ASSERT_OK(fs_->DeleteDir("bucket/somedir"));
   ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
-  ASSERT_EQ(infos.size(), 1);
-  AssertFileInfo(infos[0], "bucket/somefile", FileType::File);
+  ASSERT_EQ(infos.size(), 2);
+  AssertFileInfo(infos[0], "bucket/otherdir", FileType::Directory);
+  AssertFileInfo(infos[1], "bucket/somefile", FileType::File);
 
   // Leaving parent "directory" empty
   ASSERT_OK(fs_->CreateDir("bucket/newdir/newsub/newsubsub"));
   ASSERT_OK(fs_->DeleteDir("bucket/newdir/newsub"));
   ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
-  ASSERT_EQ(infos.size(), 2);
+  ASSERT_EQ(infos.size(), 3);
   SortInfos(&infos);
   AssertFileInfo(infos[0], "bucket/newdir", FileType::Directory);  // still exists
-  AssertFileInfo(infos[1], "bucket/somefile", FileType::File);
+  AssertFileInfo(infos[1], "bucket/otherdir", FileType::Directory);
+  AssertFileInfo(infos[2], "bucket/somefile", FileType::File);
 
   // Bucket
   ASSERT_OK(fs_->DeleteDir("bucket"));
   AssertFileInfo(fs_.get(), "bucket", FileType::NotFound);
+
+  // URI
+  ASSERT_RAISES(Invalid, fs_->DeleteDir("s3:empty-bucket"));
+}
+
+TEST_F(TestS3FS, DeleteDirContents) {
+  FileSelector select;
+  select.base_dir = "bucket";
+  std::vector<FileInfo> infos;
+
+  ASSERT_OK(fs_->DeleteDirContents("bucket/doesnotexist", /*missing_dir_ok=*/true));
+  ASSERT_OK(fs_->DeleteDirContents("bucket/emptydir"));
+  ASSERT_OK(fs_->DeleteDirContents("bucket/somedir"));
+  ASSERT_RAISES(IOError, fs_->DeleteDirContents("bucket/somefile"));
+  ASSERT_RAISES(IOError, fs_->DeleteDirContents("bucket/doesnotexist"));
+  ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
+  ASSERT_EQ(infos.size(), 4);
+  SortInfos(&infos);
+  AssertFileInfo(infos[0], "bucket/emptydir", FileType::Directory);
+  AssertFileInfo(infos[1], "bucket/otherdir", FileType::Directory);
+  AssertFileInfo(infos[2], "bucket/somedir", FileType::Directory);
+  AssertFileInfo(infos[3], "bucket/somefile", FileType::File);
+}
+
+TEST_F(TestS3FS, DeleteDirContentsAsync) {
+  FileSelector select;
+  select.base_dir = "bucket";
+  std::vector<FileInfo> infos;
+
+  ASSERT_FINISHES_OK(fs_->DeleteDirContentsAsync("bucket/emptydir"));
+  ASSERT_FINISHES_OK(fs_->DeleteDirContentsAsync("bucket/somedir"));
+  ASSERT_FINISHES_AND_RAISES(IOError, fs_->DeleteDirContentsAsync("bucket/somefile"));
+  ASSERT_OK_AND_ASSIGN(infos, fs_->GetFileInfo(select));
+  ASSERT_EQ(infos.size(), 4);
+  SortInfos(&infos);
+  AssertFileInfo(infos[0], "bucket/emptydir", FileType::Directory);
+  AssertFileInfo(infos[1], "bucket/otherdir", FileType::Directory);
+  AssertFileInfo(infos[2], "bucket/somedir", FileType::Directory);
+  AssertFileInfo(infos[3], "bucket/somefile", FileType::File);
 }
 
 TEST_F(TestS3FS, CopyFile) {
@@ -825,6 +967,9 @@ TEST_F(TestS3FS, OpenInputStream) {
   ASSERT_RAISES(IOError, fs_->OpenInputStream("nonexistent-bucket/somefile"));
   ASSERT_RAISES(IOError, fs_->OpenInputStream("bucket/zzzt"));
 
+  // URI
+  ASSERT_RAISES(Invalid, fs_->OpenInputStream("s3:bucket/somefile"));
+
   // "Files"
   ASSERT_OK_AND_ASSIGN(stream, fs_->OpenInputStream("bucket/somefile"));
   ASSERT_OK_AND_ASSIGN(buf, stream->Read(2));
@@ -880,6 +1025,9 @@ TEST_F(TestS3FS, OpenInputFile) {
   ASSERT_RAISES(IOError, fs_->OpenInputFile("nonexistent-bucket/somefile"));
   ASSERT_RAISES(IOError, fs_->OpenInputFile("bucket/zzzt"));
 
+  // URI
+  ASSERT_RAISES(Invalid, fs_->OpenInputStream("s3:bucket/somefile"));
+
   // "Files"
   ASSERT_OK_AND_ASSIGN(file, fs_->OpenInputFile("bucket/somefile"));
   ASSERT_OK_AND_EQ(9, file->GetSize());
@@ -943,6 +1091,14 @@ TEST_F(TestS3FS, OpenOutputStreamDestructorSyncWrite) {
 TEST_F(TestS3FS, OpenOutputStreamMetadata) {
   std::shared_ptr<io::OutputStream> stream;
 
+  // Create new file with no explicit or default metadata
+  // The Content-Type will still be set
+  auto empty_metadata = KeyValueMetadata::Make({}, {});
+  auto implicit_metadata =
+      KeyValueMetadata::Make({"Content-Type"}, {"application/octet-stream"});
+  AssertMetadataRoundtrip("bucket/mdfile0", empty_metadata,
+                          testing::IsSupersetOf(implicit_metadata->sorted_pairs()));
+
   // Create new file with explicit metadata
   auto metadata = KeyValueMetadata::Make({"Content-Type", "Expires"},
                                          {"x-arrow/test6", "2016-02-05T20:08:35Z"});
@@ -973,18 +1129,95 @@ TEST_F(TestS3FS, OpenOutputStreamMetadata) {
 
 TEST_F(TestS3FS, FileSystemFromUri) {
   std::stringstream ss;
-  ss << "s3://" << minio_.access_key() << ":" << minio_.secret_key()
+  ss << "s3://" << minio_->access_key() << ":" << minio_->secret_key()
      << "@bucket/somedir/subdir/subfile"
-     << "?scheme=http&endpoint_override=" << UriEscape(minio_.connect_string());
+     << "?scheme=http&endpoint_override=" << UriEscape(minio_->connect_string());
 
   std::string path;
   ASSERT_OK_AND_ASSIGN(auto fs, FileSystemFromUri(ss.str(), &path));
   ASSERT_EQ(path, "bucket/somedir/subdir/subfile");
 
+  ASSERT_OK_AND_ASSIGN(path, fs->PathFromUri(ss.str()));
+  ASSERT_EQ(path, "bucket/somedir/subdir/subfile");
+
+  // Incorrect scheme
+  ASSERT_THAT(fs->PathFromUri("file:///@bucket/somedir/subdir/subfile"),
+              Raises(StatusCode::Invalid,
+                     testing::HasSubstr("expected a URI with one of the schemes (s3)")));
+
+  // Not a URI
+  ASSERT_THAT(fs->PathFromUri("/@bucket/somedir/subdir/subfile"),
+              Raises(StatusCode::Invalid, testing::HasSubstr("Expected a URI")));
+
   // Check the filesystem has the right connection parameters
   AssertFileInfo(fs.get(), path, FileType::File, 8);
 }
 
+TEST_F(TestS3FS, NoCreateDeleteBucket) {
+  // Create a bucket to try deleting
+  ASSERT_OK(fs_->CreateDir("test-no-delete"));
+
+  options_.allow_bucket_creation = false;
+  options_.allow_bucket_deletion = false;
+  MakeFileSystem();
+
+  auto maybe_create_dir = fs_->CreateDir("test-no-create");
+  ASSERT_RAISES(IOError, maybe_create_dir);
+  ASSERT_THAT(maybe_create_dir.message(),
+              ::testing::HasSubstr("Bucket 'test-no-create' not found"));
+
+  auto maybe_delete_dir = fs_->DeleteDir("test-no-delete");
+  ASSERT_RAISES(IOError, maybe_delete_dir);
+  ASSERT_THAT(maybe_delete_dir.message(),
+              ::testing::HasSubstr("Would delete bucket 'test-no-delete'"));
+}
+
+// Simple retry strategy that records errors encountered and its emitted retry delays
+class TestRetryStrategy : public S3RetryStrategy {
+ public:
+  bool ShouldRetry(const S3RetryStrategy::AWSErrorDetail& error,
+                   int64_t attempted_retries) final {
+    errors_encountered_.emplace_back(error);
+    constexpr int64_t MAX_RETRIES = 2;
+    return attempted_retries < MAX_RETRIES;
+  }
+
+  int64_t CalculateDelayBeforeNextRetry(const S3RetryStrategy::AWSErrorDetail& error,
+                                        int64_t attempted_retries) final {
+    int64_t delay = attempted_retries;
+    retry_delays_.emplace_back(delay);
+    return delay;
+  }
+
+  std::vector<S3RetryStrategy::AWSErrorDetail> GetErrorsEncountered() {
+    return errors_encountered_;
+  }
+  std::vector<int64_t> GetRetryDelays() { return retry_delays_; }
+
+ private:
+  std::vector<S3RetryStrategy::AWSErrorDetail> errors_encountered_;
+  std::vector<int64_t> retry_delays_;
+};
+
+TEST_F(TestS3FS, CustomRetryStrategy) {
+  auto retry_strategy = std::make_shared<TestRetryStrategy>();
+  options_.retry_strategy = retry_strategy;
+  MakeFileSystem();
+  // Attempt to open file that doesn't exist. Should hit TestRetryStrategy::ShouldRetry()
+  // 3 times before bubbling back up here.
+  ASSERT_RAISES(IOError, fs_->OpenInputStream("nonexistent-bucket/somefile"));
+  ASSERT_EQ(retry_strategy->GetErrorsEncountered().size(), 3);
+  for (const auto& error : retry_strategy->GetErrorsEncountered()) {
+    ASSERT_EQ(static_cast<Aws::Client::CoreErrors>(error.error_type),
+              Aws::Client::CoreErrors::RESOURCE_NOT_FOUND);
+    ASSERT_EQ(error.message, "No response body.");
+    ASSERT_EQ(error.exception_name, "");
+    ASSERT_EQ(error.should_retry, false);
+  }
+  std::vector<int64_t> expected_retry_delays = {0, 1, 2};
+  ASSERT_EQ(retry_strategy->GetRetryDelays(), expected_retry_delays);
+}
+
 ////////////////////////////////////////////////////////////////////////////
 // Generic S3 tests
 
@@ -996,12 +1229,13 @@ class TestS3FSGeneric : public S3TestMixin, public GenericFileSystemTest {
     {
       Aws::S3::Model::CreateBucketRequest req;
       req.SetBucket(ToAwsString("s3fs-test-bucket"));
-      ASSERT_OK(OutcomeToStatus(client_->CreateBucket(req)));
+      ASSERT_OK(OutcomeToStatus("CreateBucket", client_->CreateBucket(req)));
     }
 
-    options_.ConfigureAccessKey(minio_.access_key(), minio_.secret_key());
+    options_.ConfigureAccessKey(minio_->access_key(), minio_->secret_key());
     options_.scheme = "http";
-    options_.endpoint_override = minio_.connect_string();
+    options_.endpoint_override = minio_->connect_string();
+    options_.retry_strategy = std::make_shared<ShortRetryStrategy>();
     ASSERT_OK_AND_ASSIGN(s3fs_, S3FileSystem::Make(options_));
     fs_ = std::make_shared<SubTreeFileSystem>("s3fs-test-bucket", s3fs_);
   }
