@@ -19,9 +19,9 @@
 class ApacheArrow < Formula
   desc "Columnar in-memory analytics layer designed to accelerate big data"
   homepage "https://arrow.apache.org/"
-  url "https://www.apache.org/dyn/closer.lua?path=arrow/arrow-10.0.0/apache-arrow-10.0.0.tar.gz"
+  url "https://www.apache.org/dyn/closer.lua?path=arrow/arrow-13.0.0/apache-arrow-13.0.0.tar.gz"
   sha256 "9948ddb6d4798b51552d0dca3252dd6e3a7d0f9702714fc6f5a1b59397ce1d28"
-  head "https://github.com/apache/arrow.git"
+  head "https://github.com/apache/arrow.git", branch: "main"
 
   bottle do
     cellar :any
@@ -33,8 +33,9 @@ class ApacheArrow < Formula
   depends_on "boost" => :build
   depends_on "cmake" => :build
   depends_on "aws-sdk-cpp"
-  depends_on "brotli" 
+  depends_on "brotli"
   depends_on "lz4"
+  depends_on "openssl@1.1"
   depends_on "snappy"
   depends_on "thrift"
   depends_on "zstd"
@@ -44,6 +45,7 @@ class ApacheArrow < Formula
     args = %W[
       -DARROW_BUILD_SHARED=OFF
       -DARROW_BUILD_UTILITIES=ON
+      -DARROW_ACERO=ON
       -DARROW_COMPUTE=ON
       -DARROW_CSV=ON
       -DARROW_CXXFLAGS="-D_LIBCPP_DISABLE_AVAILABILITY"
