@@ -14,6 +14,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+//go:build go1.18
+
 package exec
 
 import (
@@ -22,13 +24,15 @@ import (
 	"hash/maphash"
 	"strings"
 
-	"github.com/apache/arrow/go/v10/arrow"
-	"github.com/apache/arrow/go/v10/arrow/bitutil"
-	"github.com/apache/arrow/go/v10/arrow/internal/debug"
-	"github.com/apache/arrow/go/v10/arrow/memory"
+	"github.com/apache/arrow/go/v13/arrow"
+	"github.com/apache/arrow/go/v13/arrow/bitutil"
+	"github.com/apache/arrow/go/v13/arrow/internal/debug"
+	"github.com/apache/arrow/go/v13/arrow/memory"
 	"golang.org/x/exp/slices"
 )
 
+var hashSeed = maphash.MakeSeed()
+
 type ctxAllocKey struct{}
 
 // WithAllocator returns a new context with the provided allocator
@@ -238,6 +242,40 @@ func (primitiveMatcher) Equals(other TypeMatcher) bool {
 // returns true for.
 func Primitive() TypeMatcher { return primitiveMatcher{} }
 
+type reeMatcher struct {
+	runEndsMatcher TypeMatcher
+	encodedMatcher TypeMatcher
+}
+
+func (r reeMatcher) Matches(typ arrow.DataType) bool {
+	if typ.ID() != arrow.RUN_END_ENCODED {
+		return false
+	}
+
+	dt := typ.(*arrow.RunEndEncodedType)
+	return r.runEndsMatcher.Matches(dt.RunEnds()) && r.encodedMatcher.Matches(dt.Encoded())
+}
+
+func (r reeMatcher) Equals(other TypeMatcher) bool {
+	o, ok := other.(reeMatcher)
+	if !ok {
+		return false
+	}
+	return r.runEndsMatcher.Equals(o.runEndsMatcher) && r.encodedMatcher.Equals(o.encodedMatcher)
+}
+
+func (r reeMatcher) String() string {
+	return "run_end_encoded(run_ends=" + r.runEndsMatcher.String() + ", values=" + r.encodedMatcher.String() + ")"
+}
+
+// RunEndEncoded returns a matcher which matches a RunEndEncoded
+// type whose encoded type is matched by the passed in matcher.
+func RunEndEncoded(runEndsMatcher, encodedMatcher TypeMatcher) TypeMatcher {
+	return reeMatcher{
+		runEndsMatcher: runEndsMatcher,
+		encodedMatcher: encodedMatcher}
+}
+
 // InputKind is an enum representing the type of Input matching
 // that will be done. Either accepting any type, an exact specific type
 // or using a TypeMatcher.
