@@ -22,10 +22,11 @@ import (
 	"math/bits"
 	"reflect"
 
-	"github.com/apache/arrow/go/arrow"
-	"github.com/apache/arrow/go/arrow/memory"
-	"github.com/apache/arrow/go/parquet"
-	"github.com/apache/arrow/go/parquet/internal/utils"
+	"github.com/apache/arrow/go/v13/arrow"
+	"github.com/apache/arrow/go/v13/arrow/memory"
+	shared_utils "github.com/apache/arrow/go/v13/internal/utils"
+	"github.com/apache/arrow/go/v13/parquet"
+	"github.com/apache/arrow/go/v13/parquet/internal/utils"
 	"golang.org/x/xerrors"
 )
 
@@ -49,7 +50,8 @@ type deltaBitPackDecoder struct {
 	deltaBitWidths *memory.Buffer
 	deltaBitWidth  byte
 
-	lastVal int64
+	totalValues uint64
+	lastVal     int64
 }
 
 // returns the number of bytes read so far
@@ -84,15 +86,10 @@ func (d *deltaBitPackDecoder) SetData(nvalues int, data []byte) error {
 		return xerrors.New("parquet: eof exception")
 	}
 
-	var totalValues uint64
-	if totalValues, ok = d.bitdecoder.GetVlqInt(); !ok {
+	if d.totalValues, ok = d.bitdecoder.GetVlqInt(); !ok {
 		return xerrors.New("parquet: eof exception")
 	}
 
-	if int(totalValues) != d.nvals {
-		return xerrors.New("parquet: mismatch between number of values and count in data header")
-	}
-
 	if d.lastVal, ok = d.bitdecoder.GetZigZagVlqInt(); !ok {
 		return xerrors.New("parquet: eof exception")
 	}
@@ -159,7 +156,7 @@ func (d *DeltaBitPackInt32Decoder) unpackNextMini() error {
 // Decode retrieves min(remaining values, len(out)) values from the data and returns the number
 // of values actually decoded and any errors encountered.
 func (d *DeltaBitPackInt32Decoder) Decode(out []int32) (int, error) {
-	max := utils.MinInt(len(out), d.nvals)
+	max := shared_utils.MinInt(len(out), d.nvals)
 	if max == 0 {
 		return 0, nil
 	}
@@ -185,14 +182,13 @@ func (d *DeltaBitPackInt32Decoder) Decode(out []int32) (int, error) {
 
 		// copy as many values from our mini block as we can into out
 		start := int(d.valsPerMini - d.currentMiniBlockVals)
-		end := utils.MinInt(int(d.valsPerMini), len(out))
-		copy(out, d.miniBlockValues[start:end])
+		numCopied := copy(out, d.miniBlockValues[start:])
 
-		numCopied := end - start
 		out = out[numCopied:]
 		d.currentBlockVals -= uint32(numCopied)
 		d.currentMiniBlockVals -= uint32(numCopied)
 	}
+	d.nvals -= max
 	return max, nil
 }
 
@@ -248,7 +244,7 @@ func (d *DeltaBitPackInt64Decoder) unpackNextMini() error {
 // Decode retrieves min(remaining values, len(out)) values from the data and returns the number
 // of values actually decoded and any errors encountered.
 func (d *DeltaBitPackInt64Decoder) Decode(out []int64) (int, error) {
-	max := utils.MinInt(len(out), d.nvals)
+	max := shared_utils.MinInt(len(out), d.nvals)
 	if max == 0 {
 		return 0, nil
 	}
@@ -274,14 +270,13 @@ func (d *DeltaBitPackInt64Decoder) Decode(out []int64) (int, error) {
 		}
 
 		start := int(d.valsPerMini - d.currentMiniBlockVals)
-		end := utils.MinInt(int(d.valsPerMini), len(out))
-		copy(out, d.miniBlockValues[start:end])
+		numCopied := copy(out, d.miniBlockValues[start:])
 
-		numCopied := end - start
 		out = out[numCopied:]
 		d.currentBlockVals -= uint32(numCopied)
 		d.currentMiniBlockVals -= uint32(numCopied)
 	}
+	d.nvals -= max
 	return max, nil
 }
 
@@ -364,7 +359,7 @@ func (enc *deltaBitPackEncoder) flushBlock() {
 
 	valuesToWrite := int64(len(enc.deltas))
 	for i := 0; i < int(enc.numMiniBlocks); i++ {
-		n := utils.Min(int64(enc.miniBlockSize), valuesToWrite)
+		n := shared_utils.Min(int64(enc.miniBlockSize), valuesToWrite)
 		if n == 0 {
 			break
 		}
@@ -372,7 +367,7 @@ func (enc *deltaBitPackEncoder) flushBlock() {
 		maxDelta := int64(math.MinInt64)
 		start := i * int(enc.miniBlockSize)
 		for _, val := range enc.deltas[start : start+int(n)] {
-			maxDelta = utils.Max(maxDelta, val)
+			maxDelta = shared_utils.Max(maxDelta, val)
 		}
 
 		// compute bit width to store (max_delta - min_delta)
