@@ -26,7 +26,8 @@
 #' @importFrom rlang expr caller_env is_character quo_name is_quosure enexpr enexprs as_quosure
 #' @importFrom rlang is_list call2 is_empty as_function as_label arg_match is_symbol is_call call_args
 #' @importFrom rlang quo_set_env quo_get_env is_formula quo_is_call f_rhs parse_expr f_env new_quosure
-#' @importFrom rlang new_quosures expr_text caller_env check_dots_empty dots_list
+#' @importFrom rlang new_quosures expr_text caller_env check_dots_empty check_dots_empty0 dots_list is_string inform
+#' @importFrom rlang is_bare_list
 #' @importFrom tidyselect vars_pull eval_select eval_rename
 #' @importFrom glue glue
 #' @useDynLib arrow, .registration = TRUE
@@ -54,7 +55,13 @@ supported_dplyr_methods <- list(
   transmute = NULL,
   arrange = NULL,
   rename = NULL,
-  pull = "returns an Arrow [ChunkedArray], not an R vector",
+  pull = c(
+    "the `name` argument is not supported;",
+    "returns an R vector by default but this behavior is deprecated and will",
+    "return an Arrow [ChunkedArray] in a future release. Provide",
+    "`as_vector = TRUE/FALSE` to control this behavior, or set",
+    "`options(arrow.pull_as_vector)` globally."
+  ),
   relocate = NULL,
   compute = NULL,
   collapse = NULL,
@@ -100,6 +107,15 @@ supported_dplyr_methods <- list(
   explain = NULL
 )
 
+# This should be run at session exit and must be called
+# to avoid a segmentation fault at shutdown
+finalize_s3 <- function(env) {
+  FinalizeS3()
+}
+
+# Helper environment to register the exit hook
+s3_finalizer <- new.env(parent = emptyenv())
+
 #' @importFrom vctrs s3_register vec_size vec_cast vec_unique
 .onLoad <- function(...) {
   # Make sure C++ knows on which thread it is safe to call the R API
@@ -141,6 +157,11 @@ supported_dplyr_methods <- list(
   # Register extension types that we use internally
   reregister_extension_type(vctrs_extension_type(vctrs::unspecified()))
 
+  # Registers a callback to run at session exit
+  # This can't be done in .onUnload or .onDetach because those hooks are
+  # not guaranteed to run (e.g. they only run if the user unloads arrow)
+  reg.finalizer(s3_finalizer, finalize_s3, onexit = TRUE)
+
   invisible()
 }
 
@@ -199,6 +220,11 @@ on_linux_dev <- function() {
     grepl("devel", R.version.string)
 }
 
+on_macos_10_13_or_lower <- function() {
+  identical(unname(Sys.info()["sysname"]), "Darwin") &&
+    package_version(unname(Sys.info()["release"])) < "18.0.0"
+}
+
 option_use_threads <- function() {
   !is_false(getOption("arrow.use_threads"))
 }
